@@ -47,7 +47,7 @@ def writexml_helper(self, writer, indent="", addindent="", newl=""):
     a_names = sorted(attrs.keys())
 
     for a_name in a_names:
-        writer.write(' %s="' % a_name)
+        writer.write(f' {a_name}="')
         minidom._write_data(writer, attrs[a_name].value)
         writer.write('"')
     if self.childNodes:
@@ -69,13 +69,13 @@ def writexml_helper(self, writer, indent="", addindent="", newl=""):
             writer.write(f"</{self.tagName}>{newl}")
         else:
             # This is the normal case that we do with pretty layout
-            writer.write(">%s" % (newl))
+            writer.write(f">{newl}")
             for node in self.childNodes:
                 if node.nodeType != self.TEXT_NODE:
                     node.writexml(writer, (indent + addindent), addindent, newl)
             writer.write(f"{indent}</{self.tagName}>{newl}")
     else:
-        writer.write("/>%s" % (newl))
+        writer.write(f"/>{newl}")
 
 
 def getElementsByTagName_helper(parent, name, dummy=None):
@@ -87,7 +87,7 @@ def getElementsByTagName_helper(parent, name, dummy=None):
     yieldElementsByTagName
     """
     for node in parent.childNodes:
-        if node.nodeType == minidom.Node.ELEMENT_NODE and (name in ("*", node.tagName)):
+        if node.nodeType == minidom.Node.ELEMENT_NODE and (name in {"*", node.tagName}):
             yield node
         if node.hasChildNodes():
             yield from node.getElementsByTagName(name)
@@ -96,7 +96,7 @@ def getElementsByTagName_helper(parent, name, dummy=None):
 def searchElementsByTagName_helper(parent, name, onlysearch):
     """Limits the search to within tags occuring in onlysearch."""
     for node in parent.childNodes:
-        if node.nodeType == minidom.Node.ELEMENT_NODE and (name in ("*", node.tagName)):
+        if node.nodeType == minidom.Node.ELEMENT_NODE and (name in {"*", node.tagName}):
             yield node
         if node.nodeType == minidom.Node.ELEMENT_NODE and node.tagName in onlysearch:
             for node in node.searchElementsByTagName(name, onlysearch):
@@ -222,7 +222,7 @@ class ExpatBuilderNS(expatbuilder.ExpatBuilderNS):
                     )
                     a = minidom.Attr(qname, uri, localname, prefix)
                     _attrs[qname] = a
-                    _attrsNS[(uri, localname)] = a
+                    _attrsNS[uri, localname] = a
                 else:
                     a = minidom.Attr(
                         aname,
@@ -231,7 +231,7 @@ class ExpatBuilderNS(expatbuilder.ExpatBuilderNS):
                         expatbuilder.EMPTY_PREFIX,
                     )
                     _attrs[aname] = a
-                    _attrsNS[(expatbuilder.EMPTY_NAMESPACE, aname)] = a
+                    _attrsNS[expatbuilder.EMPTY_NAMESPACE, aname] = a
                 a.ownerDocument = self.document
                 a.value = value
                 a.ownerElement = node
