@@ -128,7 +128,7 @@ class UtxUnit(base.TranslationUnit):
 
     def addnote(self, text, origin=None, position="append"):
         currentnote = self._get_field("comment")
-        if position == "append" and currentnote is not None and currentnote != "":
+        if position == "append" and currentnote:
             self._set_field("comment", currentnote + "\n" + text)
         else:
             self._set_field("comment", text)
@@ -228,7 +228,7 @@ class UtxFile(base.TranslationStore):
         )
         items = []
         for key, value in self._header.items():
-            if key in ["version", "source_language", "target_language", "date_created"]:
+            if key in {"version", "source_language", "target_language", "date_created"}:
                 continue
             items.append(f"{key}: {value}")
         if len(items):
